#ifndef WMAINTOOLBAR_H
#define WMAINTOOLBAR_H

#include <QToolBar>
#include <QActionGroup>
#include <QGraphicsItem>
#include <QPainter>
#include <QPaintEvent>
#include <QStyleOption>

class wMainToolBar : public QToolBar
{
    Q_OBJECT
public:
    explicit wMainToolBar(QWidget *parent = nullptr);//关键字 explicit 可以禁止“单参数构造函数”被用于自动类型转换

signals:
    void sigCreateElement(int);
    void sigSave();
    void sigSaveAs();
    void sigSetting();
    void sigSend();
    void sigPreview(int);

public slots:
    void onActionTriggered(QAction *act);
    void OnProgPlayerQuit();
private:
    int m_flagPreviewing=0;
};

#endif // WMAINTOOLBAR_H
